package com.ruoyi.zdsz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.annotation.DynamicName;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.framework.obs.ObsService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.zdsz.domain.bo.ZEngiineeringPhotoBo;
import com.ruoyi.zdsz.domain.vo.ZEngiineeringPhotoVo;
import com.ruoyi.zdsz.domain.ZEngiineeringPhoto;
import com.ruoyi.zdsz.mapper.ZEngiineeringPhotoMapper;
import com.ruoyi.zdsz.service.IZEngiineeringPhotoService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Collection;
import java.util.stream.Collectors;

/**
 * 照片Service业务层处理
 *
 * @author ruoyi
 * @date 2023-12-27
 */
@RequiredArgsConstructor
@Service
public class ZEngiineeringPhotoServiceImpl implements IZEngiineeringPhotoService {

    private final ZEngiineeringPhotoMapper baseMapper;
    @Resource
   private ObsService obsService;

    /**
     * 查询照片
     */
    @Override
    public ZEngiineeringPhotoVo queryById(String id){
        return baseMapper.selectVoById(id);
    }

    @Override
    public List<String> queryList(String id) {
        List<String> list =new ArrayList<>();
        LambdaQueryWrapper<ZEngiineeringPhoto> lqw = Wrappers.lambdaQuery();
        lqw.eq(StringUtils.isNotBlank(id), ZEngiineeringPhoto::getParentId, id);
        List<ZEngiineeringPhoto> zEngiineeringPhotoList= baseMapper.selectList(lqw);
        zEngiineeringPhotoList.stream().forEach( item->{
            list.add(item.getPicUrl());
        });
        return list;
    }

    /**
     * 查询照片列表
     */
    @Override
    public TableDataInfo<ZEngiineeringPhotoVo> queryPageList(ZEngiineeringPhotoBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZEngiineeringPhoto> lqw = buildQueryWrapper(bo);
        Page<ZEngiineeringPhotoVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询照片列表
     */
    @Override
    public List<ZEngiineeringPhotoVo> queryList(ZEngiineeringPhotoBo bo) {
        LambdaQueryWrapper<ZEngiineeringPhoto> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZEngiineeringPhoto> buildQueryWrapper(ZEngiineeringPhotoBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZEngiineeringPhoto> lqw = Wrappers.lambdaQuery();
        lqw.eq(StringUtils.isNotBlank(bo.getParentId()), ZEngiineeringPhoto::getParentId, bo.getParentId());
        lqw.eq(StringUtils.isNotBlank(bo.getPicUrl()), ZEngiineeringPhoto::getPicUrl, bo.getPicUrl());
        return lqw;
    }

    /**
     * 新增照片
     */
    @Override
    public Boolean insertByBo(ZEngiineeringPhotoBo bo) {
        ZEngiineeringPhoto add = BeanUtil.toBean(bo, ZEngiineeringPhoto.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 批量插入
     * @param bo
     * @return
     */
    @Override
    @Transactional(rollbackFor=Exception.class)
    @DynamicName(spel = "#bo[0].createTime")
    public Boolean insertBatch( List<ZEngiineeringPhoto> bo) {
        boolean flag = baseMapper.insertBatch(bo);
        return flag;
    }

    /**
     * 修改照片
     */
    @Override
    public Boolean updateByBo(ZEngiineeringPhotoBo bo) {
        ZEngiineeringPhoto update = BeanUtil.toBean(bo, ZEngiineeringPhoto.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    @Override
    @DynamicName(spel = "#bo[0].createTime")
    public Boolean updateBatch(List<ZEngiineeringPhoto> bo) {
        ZEngiineeringPhoto zEngiineeringPhoto = bo.get(0);
        LambdaQueryWrapper<ZEngiineeringPhoto> lqw = Wrappers.lambdaQuery();
        lqw.eq(StringUtils.isNotBlank(zEngiineeringPhoto.getParentId()), ZEngiineeringPhoto::getParentId, zEngiineeringPhoto.getParentId());
        List<ZEngiineeringPhotoVo> zEngiineeringPhotoList=  baseMapper.selectVoList(lqw);
        List<String> dlList=new ArrayList<>();
        List<String> dlIdList=new ArrayList<>();
        List<ZEngiineeringPhoto> instList=new ArrayList<>();
         zEngiineeringPhotoList.stream().forEach(item->{
           bo.stream().forEach(boitem->{
               if(item.getPicUrl()!= boitem.getPicUrl())
               {
                   dlList.add(item.getPicUrl());
                   dlIdList.add(item.getId());
                   instList.add(boitem);
               }
           });

        });
        obsService.deleteFiles(dlList);
        baseMapper.deleteBatchIds(dlIdList);
        return  insertBatch(instList);
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZEngiineeringPhoto entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除照片
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<String> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
